//------------------------------------------------------------------------
//  Name      | TQRDecisionGrid
//  Note      | Component for building multi-dimensional data reports
//  File      | QRDGrid.pas
//  Platform  | Delphi; tested on versions 3.0, 4.0, 5.0
//  Version   | v.1.01, 02.11.2000
//  Author    | Yuri A. Smantzerr, commonaspects@mail.com
//
//------------------------------------------------------------------------
//
//  This component is FREEWARE and comes with full source code.
//  However, I still hold the copyright.
//  If you made any modifications to the code, please send it to me. 
//  If you like this component or have any ideas for improvement
//  or bug reports, don't hesitate to send me a mail.
//
//  This component allow you to build multidimensional reports without
//  coding. It`s important when you use a report editor and don`t want
//  to have the code in report's unit.
//  If you find a problems or solutions, give me to know.
//  I do not guarantee support but I will try to solve
//  problems as is it possible.
//  Yu need to use TDataSet descendant component which have only one 
//  record to initialize report generation if you don`t want
//  to have manual coding (see demo).
//
//------------------------------------------------------------------------
//  revision history
//
//  24.03.2000 version 1.0  released
//  02.11.2000 version 1.01 fix problem with stretching on print
//------------------------------------------------------------------------

unit QRDGrid;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  QuickRpt, MXDB, mxgrid, DB, Grids;

type

  { TQRDecisionGrid }

  TQRDecisionGrid = class(TQRPrintable)
  private
    { Private declarations }
    FDecisionGrid: TDecisionGrid;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    function GetDecisionSource: TDecisionSource;
    procedure SetDecisionSource(Value: TDecisionSource);
    function GetOptions: TDecisionGridOptions;
    procedure SetOptions(Value: TDecisionGridOptions);
    function GetBorderStyle: TBorderStyle;
    procedure SetBorderStyle(Value: TBorderStyle);
    function GetCaptionColor: TColor;
    procedure SetCaptionColor(Value: TColor);
    function GetCaptionFont: TFont;
    procedure SetCaptionFont(Value: TFont);
    function GetColor: TColor;
    procedure SetColor(Value: TColor);
    function GetDataColor: TColor;
    procedure SetDataColor(Value: TColor);
    function GetDataFont: TFont;
    procedure SetDataFont(Value: TFont);
    function GetDataSumColor: TColor;
    procedure SetDataSumColor(Value: TColor);
    function GetDefaultColWidth: Integer;
    procedure SetDefaultColWidth(Value: Integer);
    function GetDefaultDrawing: Boolean;
    procedure SetDefaultDrawing(Value: Boolean);
    function GetDefaultRowHeight: Integer;
    procedure SetDefaultRowHeight(Value: Integer);
    function GetDimensions: TDisplayDims;
    procedure SetDimensions(Value: TDisplayDims);
    function GetGridLineColor: TColor;
    procedure SetGridLineColor(Value: TColor);
    function GetGridLineWidth: Integer;
    procedure SetGridLineWidth(Value: Integer);
    function GetLabelColor: TColor;
    procedure SetLabelColor(Value: TColor);
    function GetLabelFont: TFont;
    procedure SetLabelFont(Value: TFont);
    function GetLabelSumColor: TColor;
    procedure SetLabelSumColor(Value: TColor);
    function GetTotals: Boolean;
    procedure SetTotals(Value: Boolean);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    procedure Paint; override;
    procedure Print(OfsX, OfsY : integer); override;
  published
    { Published declarations }
    property BorderStyle: TBorderStyle read GetBorderStyle write SetBorderStyle default bsNone;
    property CaptionColor: TColor read GetCaptionColor write SetCaptionColor default clWhite;
    property CaptionFont: TFont read GetCaptionFont write SetCaptionFont;
    property Color: TColor read GetColor write SetColor default clWhite;
    property DataColor: TColor read GetDataColor write SetDataColor default clWhite;
    property DataFont: TFont read GetDataFont write SetDataFont;
    property DataSumColor: TColor read GetDataSumColor write SetDataSumColor default clWindow;
    property DecisionSource: TDecisionSource read GetDecisionSource write SetDecisionSource;
    property DefaultColWidth: Integer read GetDefaultColWidth write SetDefaultColWidth default 70;
    property DefaultDrawing: Boolean read GetDefaultDrawing write SetDefaultDrawing default True;
    property DefaultRowHeight: Integer read GetDefaultRowHeight write SetDefaultRowHeight default 20;
    property Dimensions: TDisplayDims read GetDimensions write SetDimensions;
    property GridLineColor: TColor read GetGridLineColor write SetGridLineColor default clBlack;
    property GridLineWidth: Integer read GetGridLineWidth write SetGridLineWidth default 1;
    property LabelColor: TColor read GetLabelColor write SetLabelColor default clWindow;
    property LabelFont: TFont read GetLabelFont write SetLabelFont;
    property LabelSumColor: TColor read GetLabelSumColor write SetLabelSumColor default clWindow;
    property Options: TDecisionGridOptions read GetOptions write SetOptions default [cgGridLines];
    property Totals: Boolean read GetTotals write SetTotals default False;
  end;

procedure Register;

implementation

uses
  StdCtrls;

procedure Register;
begin
  RegisterComponents('QReport', [TQRDecisionGrid]);
end;

procedure TQRDecisionGrid.WMSize(var Message: TWMSize);
begin
  inherited;
  FDecisionGrid.Width := Width;
  FDecisionGrid.Height := Height;
end;

constructor TQRDecisionGrid.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Width := 200;
  Height := 100;
  FDecisionGrid := TDecisionGrid.Create(Self);
  FDecisionGrid.Parent := Self;
  FDecisionGrid.ScrollBars := ssBoth;
  FDecisionGrid.Enabled := False;
  Options := [cgGridLines];
  BorderStyle := bsNone;
  CaptionColor := clWhite;
  Color := clWhite;
  DataColor := clWhite;
  DataSumColor := clWindow;
  DefaultColWidth := 70;
  DefaultDrawing := True;
  DefaultRowHeight := 20;
  GridLineColor := clBlack;
  GridLineWidth := 1;
  LabelColor := clWindow;
  LabelSumColor := clWindow;
  Totals := False;
end;

procedure TQRDecisionGrid.Paint;
begin
  FDecisionGrid.PaintTo(Canvas.Handle, 0, 0);
end;

procedure TQRDecisionGrid.Print(OfsX, OfsY : integer);
var
  Dest : TRect;
  MetaFile: TMetaFile;
  MetaCanvas: TMetafileCanvas;
begin
  // correct size
  FDecisionGrid.Width := Width;
  FDecisionGrid.Height := Height;

  // define anchor's points
  Dest.Left := QRPrinter.XPos(OfsX + Size.Left);
  Dest.Top := QRPrinter.YPos(OfsY + Size.Top);
  Dest.Right := QRPrinter.XPos(OfsX + Size.Left + Size.Width);
  Dest.Bottom := QRPrinter.XPos(OfsY + Size.Top + Size.Height);

  try
    MetaFile := TMetaFile.Create;
    MetaFile.Width := Width;
    MetaFile.Height := Height;
    MetaCanvas := TMetaFileCanvas.Create(MetaFile, 0);
    FDecisionGrid.PaintTo(MetaCanvas.Handle, 0, 0);
    MetaCanvas.Free;

    QRPrinter.Canvas.StretchDraw(Dest, MetaFile);
  finally
    MetaFile.Free;
  end;
end;

function TQRDecisionGrid.GetDecisionSource: TDecisionSource;
begin
  Result := FDecisionGrid.DecisionSource;
end;

procedure TQRDecisionGrid.SetDecisionSource(Value: TDecisionSource);
begin
  FDecisionGrid.DecisionSource := Value;
end;

function TQRDecisionGrid.GetOptions: TDecisionGridOptions;
begin
  Result := FDecisionGrid.Options;
end;

procedure TQRDecisionGrid.SetOptions(Value: TDecisionGridOptions);
begin
  FDecisionGrid.Options := Value;
end;

function TQRDecisionGrid.GetBorderStyle: TBorderStyle;
begin
  Result := FDecisionGrid.BorderStyle;
end;

procedure TQRDecisionGrid.SetBorderStyle(Value: TBorderStyle);
begin
  FDecisionGrid.BorderStyle := Value;
end;

function TQRDecisionGrid.GetCaptionColor: TColor;
begin
  Result := FDecisionGrid.CaptionColor;
end;

procedure TQRDecisionGrid.SetCaptionColor(Value: TColor);
begin
  FDecisionGrid.CaptionColor := Value;
end;

function TQRDecisionGrid.GetCaptionFont: TFont;
begin
  Result := FDecisionGrid.CaptionFont;
end;

procedure TQRDecisionGrid.SetCaptionFont(Value: TFont);
begin
  FDecisionGrid.CaptionFont := Value;
end;

function TQRDecisionGrid.GetColor: TColor;
begin
  Result := FDecisionGrid.Color;
end;

procedure TQRDecisionGrid.SetColor(Value: TColor);
begin
  FDecisionGrid.Color := Value;
end;

function TQRDecisionGrid.GetDataColor: TColor;
begin
  Result := FDecisionGrid.DataColor;
end;

procedure TQRDecisionGrid.SetDataColor(Value: TColor);
begin
  FDecisionGrid.DataColor := Value;
end;

function TQRDecisionGrid.GetDataFont: TFont;
begin
  Result := FDecisionGrid.DataFont;
end;

procedure TQRDecisionGrid.SetDataFont(Value: TFont);
begin
  FDecisionGrid.DataFont := Value;
end;

function TQRDecisionGrid.GetDataSumColor: TColor;
begin
  Result := FDecisionGrid.DataSumColor;
end;

procedure TQRDecisionGrid.SetDataSumColor(Value: TColor);
begin
  FDecisionGrid.DataSumColor := Value;
end;

function TQRDecisionGrid.GetDefaultColWidth: Integer;
begin
  Result := FDecisionGrid.DefaultColWidth;
end;

procedure TQRDecisionGrid.SetDefaultColWidth(Value: Integer);
begin
  FDecisionGrid.DefaultColWidth := Value;
end;

function TQRDecisionGrid.GetDefaultDrawing: Boolean;
begin
  Result := FDecisionGrid.DefaultDrawing;
end;

procedure TQRDecisionGrid.SetDefaultDrawing(Value: Boolean);
begin
  FDecisionGrid.DefaultDrawing := Value;
end;

function TQRDecisionGrid.GetDefaultRowHeight: Integer;
begin
  Result := FDecisionGrid.DefaultRowHeight;
end;

procedure TQRDecisionGrid.SetDefaultRowHeight(Value: Integer);
begin
  FDecisionGrid.DefaultRowHeight := Value;
end;

function TQRDecisionGrid.GetDimensions: TDisplayDims;
begin
  Result := FDecisionGrid.Dimensions;
end;

procedure TQRDecisionGrid.SetDimensions(Value: TDisplayDims);
begin
  FDecisionGrid.Dimensions := Value;
end;

function TQRDecisionGrid.GetGridLineColor: TColor;
begin
  Result := FDecisionGrid.GridLineColor;
end;

procedure TQRDecisionGrid.SetGridLineColor(Value: TColor);
begin
  FDecisionGrid.GridLineColor := Value;
end;

function TQRDecisionGrid.GetGridLineWidth: Integer;
begin
  Result := FDecisionGrid.GridLineWidth;
end;

procedure TQRDecisionGrid.SetGridLineWidth(Value: Integer);
begin
  FDecisionGrid.GridLineWidth := Value;
end;

function TQRDecisionGrid.GetLabelColor: TColor;
begin
  Result := FDecisionGrid.LabelColor;
end;

procedure TQRDecisionGrid.SetLabelColor(Value: TColor);
begin
  FDecisionGrid.LabelColor := Value;
end;

function TQRDecisionGrid.GetLabelFont: TFont;
begin
  Result := FDecisionGrid.LabelFont;
end;

procedure TQRDecisionGrid.SetLabelFont(Value: TFont);
begin
  FDecisionGrid.LabelFont := Value;
end;

function TQRDecisionGrid.GetLabelSumColor: TColor;
begin
  Result := FDecisionGrid.LabelSumColor;
end;

procedure TQRDecisionGrid.SetLabelSumColor(Value: TColor);
begin
  FDecisionGrid.LabelSumColor := Value;
end;

function TQRDecisionGrid.GetTotals: Boolean;
begin
  Result := FDecisionGrid.Totals;
end;

procedure TQRDecisionGrid.SetTotals(Value: Boolean);
begin
  FDecisionGrid.Totals := Value;
end;

end.
